@@ -31,7 +31,6 @@
 ;;; Code:
 
 (require 'comint)
-(require 'pulse)
 (require 'neocaml)
 
 (defgroup neocaml-repl nil
@@ -57,29 +56,9 @@
   :group 'neocaml-repl
   :package-version '(neocaml . "0.1.0"))
 
-(defcustom neocaml-repl-history-file
-  (expand-file-name "neocaml-repl-history" user-emacs-directory)
-  "File to persist OCaml REPL input history across sessions.
-Set to nil to disable history persistence."
-  :type '(choice (file :tag "History file")
-                 (const :tag "Disable" nil))
-  :group 'neocaml-repl
-  :package-version '(neocaml . "0.2.0"))
-
-(defcustom neocaml-repl-history-size 1000
-  "Maximum number of input history entries to persist."
-  :type 'integer
-  :group 'neocaml-repl
-  :package-version '(neocaml . "0.2.0"))
-
-(defvar-local neocaml-repl--source-buffer nil
-  "Source buffer from which the REPL was last invoked.
-Used by `neocaml-repl-switch-to-source' to return to the source buffer.")
-
 (defvar neocaml-repl-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map comint-mode-map)
-    (define-key map (kbd "C-c C-z") #'neocaml-repl-switch-to-source)
     map)
   "Keymap for `neocaml-repl-mode'.")
 
@@ -92,8 +71,8 @@ Used by `neocaml-repl-switch-to-source' to return to the source buffer.")
     ;; values and types from result
     ("\\(val\\) \\([^:]*\\)\\( *:\\)" (1 font-lock-keyword-face) (2 font-lock-variable-name-face))
     ("\\(type\\) \\([^ =]*\\)" (1 font-lock-keyword-face) (2 font-lock-type-face))
-    ;; prompt (standard "# " and utop's "utop # " / "utop[0] # ")
-    ("^\\(utop\\(\\[[0-9]+\\]\\)? \\)?# " . font-lock-comment-face))
+    ;; prompt
+    ("^# " . font-lock-comment-face))
   "Font-lock keywords for the OCaml REPL buffer.
 Highlights prompts, errors, warnings, and toplevel response values.")
 
@@ -101,13 +80,10 @@ Highlights prompts, errors, warnings, and toplevel response values.")
   "Major mode for interacting with an OCaml toplevel.
 
 \\{neocaml-repl-mode-map}"
-  ;; Match both the standard "# " prompt and utop's "utop # " / "utop[0] # "
-  (setq comint-prompt-regexp "^\\(utop\\(\\[[0-9]+\\]\\)? \\)?# ")
+  (setq comint-prompt-regexp "^# ")
   (setq comint-prompt-read-only t)
   (setq comint-input-sender 'neocaml-repl--input-sender)
   (setq comint-process-echoes nil)
-  ;; Strip ANSI escape sequences emitted by utop and other enhanced toplevels
-  (ansi-color-for-comint-mode-on)
   (setq-local comment-start "(* ")
   (setq-local comment-end " *)")
   (setq-local comment-start-skip "(\\*+[ \t]*")
@@ -119,30 +95,15 @@ Highlights prompts, errors, warnings, and toplevel response values.")
                 ("^\\(Warning\\|Alert\\): \\(.+\\), line \\([0-9]+\\), characters \\([0-9]+\\)-\\([0-9]+\\)" 2 3 4 1)))
   (compilation-shell-minor-mode)
 
-  ;; Input history persistence
-  (when neocaml-repl-history-file
-    (setq comint-input-ring-file-name neocaml-repl-history-file)
-    (setq comint-input-ring-size neocaml-repl-history-size)
-    (setq comint-input-ignoredups t)
-    (comint-read-input-ring t)
-    (add-hook 'kill-buffer-hook #'comint-write-input-ring nil t))
-
   ;; Setup prettify-symbols (users enable prettify-symbols-mode via hooks)
-  (setq-local prettify-symbols-alist
-              (if neocaml-prettify-symbols-full
-                  (append neocaml-prettify-symbols-alist
-                          neocaml-prettify-symbols-extra-alist)
-                neocaml-prettify-symbols-alist)))
+  (setq-local prettify-symbols-alist neocaml-prettify-symbols-alist))
 
 (defun neocaml-repl--input-sender (proc input)
-  "Send INPUT to PROC, appending `;;' terminator if missing.
-Only checks whether the input ends with `;;' (ignoring trailing whitespace),
-to avoid false positives from `;;' inside strings or comments."
-  (let* ((trimmed (string-trim-right input))
-         (terminated (string-suffix-p ";;" trimmed)))
-    (comint-send-string proc (if terminated
-                                 (concat input "\n")
-                               (concat input "\n;;\n")))))
+  "Send INPUT to PROC, appending `;;' terminator if missing."
+  (let ((input-with-terminator (if (string-match-p ";;" (string-trim input))
+                                   input
+                                 (concat input "\n;;"))))
+    (comint-send-string proc (concat input-with-terminator "\n"))))
 
 ;;;###autoload
 (defun neocaml-repl-start ()
@@ -158,34 +119,22 @@ If a process is already running, switch to its buffer."
         (neocaml-repl-mode))
       (pop-to-buffer buffer))))
 
-(defun neocaml-repl-switch-to-source ()
-  "Switch from the REPL back to the source buffer that last invoked it."
-  (interactive)
-  (if (and neocaml-repl--source-buffer
-           (buffer-live-p neocaml-repl--source-buffer))
-      (pop-to-buffer neocaml-repl--source-buffer)
-    (message "No source buffer to return to")))
-
 ;;;###autoload
 (defun neocaml-repl-switch-to-repl ()
-  "Switch to the OCaml REPL, saving the current buffer as the source.
-If a REPL is already running, switch to it; otherwise start a new one.
-Use \\[neocaml-repl-switch-to-source] in the REPL to return."
+  "Switch to the OCaml REPL buffer if it exists, otherwise start a new one."
   (interactive)
-  (let ((source (current-buffer)))
-    (if (get-buffer neocaml-repl-buffer-name)
-        (pop-to-buffer neocaml-repl-buffer-name)
-      (neocaml-repl-start))
-    (setq neocaml-repl--source-buffer source)))
+  (if (get-buffer neocaml-repl-buffer-name)
+      (pop-to-buffer neocaml-repl-buffer-name)
+    (neocaml-repl-start)))
 
 ;;;###autoload
 (defun neocaml-repl-send-region (start end)
   "Send the region between START and END to the OCaml REPL."
   (interactive "r")
-  (let ((text (buffer-substring-no-properties start end)))
+  (let ((region (buffer-substring-no-properties start end)))
     (neocaml-repl--ensure-repl-running)
-    (neocaml-repl--input-sender (neocaml-repl--process) text)
-    (pulse-momentary-highlight-region start end)))
+    (comint-send-string (get-buffer-process neocaml-repl-buffer-name) region)
+    (comint-send-string (get-buffer-process neocaml-repl-buffer-name) "\n;;\n")))
 
 ;;;###autoload
 (defun neocaml-repl-send-buffer ()
@@ -197,54 +146,34 @@ Use \\[neocaml-repl-switch-to-source] in the REPL to return."
 (defun neocaml-repl-send-definition ()
   "Send the current definition to the OCaml REPL."
   (interactive)
-  (if-let* ((node (treesit-defun-at-point))
-            (start (treesit-node-start node))
-            (end (treesit-node-end node)))
-      (neocaml-repl-send-region start end)
-    (user-error "No definition at point")))
-
-(defun neocaml-repl--search-phrase-delimiter (direction)
-  "Search for a `;;' phrase delimiter in DIRECTION (1 or -1).
-Skips matches inside strings and comments.  Returns the position
-after the delimiter (forward) or before it (backward), or nil."
-  (let ((search-fn (if (> direction 0) #'search-forward #'search-backward))
-        (found nil))
-    (save-excursion
-      (while (and (not found)
-                  (funcall search-fn ";;" nil t))
-        (let ((state (syntax-ppss)))
-          (unless (or (nth 3 state) (nth 4 state))
-            (setq found (if (> direction 0) (point) (match-beginning 0)))))))
-    found))
+  (let* ((node (treesit-defun-at-point))
+         (start (treesit-node-start node))
+         (end (treesit-node-end node)))
+    (when (and start end)
+      (neocaml-repl-send-region start end))))
 
 ;;;###autoload
 (defun neocaml-repl-send-phrase ()
-  "Send the current phrase (code between `;;' delimiters) to the OCaml REPL.
-Skips `;;' that appear inside strings or comments."
+  "Send the current phrase (code up to next ;;) to the OCaml REPL."
   (interactive)
-  (let ((end (or (neocaml-repl--search-phrase-delimiter 1) (point-max)))
-        (start (or (neocaml-repl--search-phrase-delimiter -1) (point-min))))
-    (neocaml-repl-send-region start end)))
-
-(defun neocaml-repl--process ()
-  "Return the REPL process, or nil if not running."
-  (get-buffer-process neocaml-repl-buffer-name))
+  (save-excursion
+    (let ((end (save-excursion
+                 (if (search-forward ";;" nil t)
+                     (point)
+                   (goto-char (point-max))
+                   (point))))
+          (start (save-excursion
+                   (if (search-backward ";;" nil t)
+                       (match-end 0)
+                     (goto-char (point-min))
+                     (point)))))
+      (neocaml-repl-send-region start end))))
 
 (defun neocaml-repl--ensure-repl-running ()
   "Start an OCaml REPL if one is not already running."
   (unless (comint-check-proc neocaml-repl-buffer-name)
     (neocaml-repl-start)))
 
-;;;###autoload
-(defun neocaml-repl-load-file (file)
-  "Load FILE into the OCaml REPL via the `#use' directive."
-  (interactive (list (buffer-file-name)))
-  (unless file
-    (user-error "Buffer is not visiting a file"))
-  (neocaml-repl--ensure-repl-running)
-  (neocaml-repl--input-sender (neocaml-repl--process)
-                               (format "#use %S" file)))
-
 (defun neocaml-repl-clear-buffer ()
   "Clear the OCaml REPL buffer."
   (interactive)
@@ -257,7 +186,7 @@ Skips `;;' that appear inside strings or comments."
   "Interrupt the OCaml REPL process."
   (interactive)
   (when (comint-check-proc neocaml-repl-buffer-name)
-    (interrupt-process (neocaml-repl--process))))
+    (interrupt-process (get-buffer-process neocaml-repl-buffer-name))))
 
 ;; Installation of the toplevel integration
 (defvar neocaml-repl-mode-hook nil
@@ -270,7 +199,6 @@ Skips `;;' that appear inside strings or comments."
     (define-key map (kbd "C-c C-r") #'neocaml-repl-send-region)
     (define-key map (kbd "C-c C-b") #'neocaml-repl-send-buffer)
     (define-key map (kbd "C-c C-p") #'neocaml-repl-send-phrase)
-    (define-key map (kbd "C-c C-l") #'neocaml-repl-load-file)
     (define-key map (kbd "C-c C-i") #'neocaml-repl-interrupt)
     (define-key map (kbd "C-c C-k") #'neocaml-repl-clear-buffer)
 
@@ -282,7 +210,6 @@ Skips `;;' that appear inside strings or comments."
         ["Send Region" neocaml-repl-send-region]
         ["Send Buffer" neocaml-repl-send-buffer]
         ["Send Phrase" neocaml-repl-send-phrase]
-        ["Load File" neocaml-repl-load-file]
         "--"
         ["Interrupt REPL" neocaml-repl-interrupt]
         ["Clear REPL Buffer" neocaml-repl-clear-buffer]))
